%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%     Simulation for endowment economy    %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This file loads value and policy functions generated by MPPsolve to
% simulate the endowment economy. Then proceeds to calculate moments

clc
%%%%%%%%%%%%% Parameters %%%%%%%%%%%%%%%%
T=200000;
cut=1000;  % Initial condition dependence

%% 1. Allocates Grids and initial values for simulation
load MPPpolicies.mat        % Loads previous results

% Uses the big transition matrix to generate a simulation of (T+cut)
% periods.
[S_index,~]=markov(Prob,T+cut+1,1,1:length(Prob));

% Initialize vectors for simulation (decentralized equilibrium)
bpSIM=zeros(T+cut,1);       % Next period Debt
WfSIM=zeros(T+cut,1);       % Welfare gain
bindSIM=zeros(T+cut,1);     % Indicator of binding debt constraint
cSIM=zeros(T+cut,1);        % Consumption
pSIM=zeros(T+cut,1);        % Non-tradables price

ySIM=zeros(T+cut,1);        % Output
RSIM=zeros(T+cut,1);        % Interest Rate

% Initialize vectors for Social Planner simulation
bpSP_SIM=zeros(T+cut,1);    % Next period Debt
bindSP_SIM=zeros(T+cut,1);  % Indicator of binding constraint
cSP_SIM=zeros(T+cut,1);     % Consumption
pSP_SIM=zeros(T+cut,1);     % Non-tradables price

Tau_SIM=zeros(T+cut,1);     % Tax




bpSIM(1)=bmax+(bmin-bmax)/2;        % t=0 period debt
bpSP_SIM(1)=bmax+(bmin-bmax)/2;     % t=0 period debt for Social Planner

ySIM=yt(1,S_index)';        % Simulated output path
RSIM=R(1,S_index)';         % Simulated interest rate path

%% 2 Recursive evolution of endogenous variables
for i=2:T+cut
    
    % Decentralized equilibrium simulations
    % Get next period debt (i+1) given by debt policy (bp) and current debt
    % chosen last period: bpSIM(i-i) is period (i) debt.
    bpSIM(i)=interp1(B,bp(:,S_index(i)),bpSIM(i-1),'linear');
    % Consumption equivalent welfare gain
    WfSIM(i)=interp1(B,Wf(:,S_index(i)),bpSIM(i-1),'linear');
    % Get current consumption from current state (output, interest rate and
    % current debt) and new debt
    cSIM(i)=bpSIM(i-1)+ySIM(i)-(1/(1+RSIM(i)))*bpSIM(i); 
    
    % Social planner simulations
    % Next period debt given current state
    bpSP_SIM(i)=interp1(B,bpSP(:,S_index(i)),bpSP_SIM(i-1),'linear');
    % Consumption from budget constraint
    cSP_SIM(i)=bpSP_SIM(i-1)+ySIM(i)-(1/(1+RSIM(i)))*bpSP_SIM(i);
    % Optimal Macro-Prudential tax per period
    Tau_SIM(i)=interp1(B,Tau(:,S_index(i)),bpSP_SIM(i-1),'linear');
end

% Get Nontradable prices that clear markets
pSIM=(1-omega)/omega*cSIM.^(1+ita);         % Decentralized Equilibrium
pSP_SIM=(1-omega)/omega*cSP_SIM.^(1+ita);   % Social Planner's

% Debt limits:
bplim_SIM=-(1+RSIM).*kappa.*(pSIM+ySIM);    
bplimSP_SIM=-(1+RSIM).*kappa.*(pSP_SIM+ySIM);

% Identify if the constraint was binding:
%%%%% IMPORTANT TO USE TOLERANCE NOT <= %%%%%%%%%%%%
bindSIM((bpSIM-bplim_SIM)<=1e-5)=1;
bindSP_SIM((bpSP_SIM-bplimSP_SIM)<=1e-5)=1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Asign debt to limits when binding 
bpSIM(bindSIM==1)=bplim_SIM(bindSIM==1);
bpSP_SIM(bindSP_SIM==1)=bplimSP_SIM(bindSP_SIM==1);

% Current Account = change in NFA = change in debt
CA_SIM=bpSIM(2:end)-bpSIM(1:end-1);  
CASP_SIM=bpSP_SIM(2:end)-bpSP_SIM(1:end-1);

% Eliminate Initial Condition Dependency
S_index=S_index(cut+1:T+cut)';
bpSIM=bpSIM(cut+1:T+cut);
bindSIM=bindSIM(cut+1:T+cut);
bplim_SIM=bplim_SIM(cut+1:T+cut);
cSIM=cSIM(cut+1:T+cut);
pSIM=pSIM(cut+1:T+cut);
CA_SIM=CA_SIM(cut:T+cut-1);

bpSP_SIM=bpSP_SIM(cut+1:T+cut);
bindSP_SIM=bindSP_SIM(cut+1:T+cut);
bplimSP_SIM=bplimSP_SIM(cut+1:T+cut);
cSP_SIM=cSP_SIM(cut+1:T+cut);
pSP_SIM=pSP_SIM(cut+1:T+cut);
CASP_SIM=CASP_SIM(cut:T+cut-1);

Tau_SIM=Tau_SIM(cut+1:T+cut);
ySIM=ySIM(cut+1:T+cut);
RSIM=RSIM(cut+1:T+cut);

%% 3. Aggregate Moments
% Total Income: tradables + nontradables
Y_SIM=pSIM+ySIM;   
YSP_SIM=pSP_SIM+ySIM;

% Current Account / GDP
CA_SIM=CA_SIM./Y_SIM; 
CASP_SIM=CASP_SIM./YSP_SIM;
% Change in CurrentAccount / GDP indicator
CAchg_SIM=CA_SIM(2:end)-CA_SIM(1:end-1);
CAchg_SIM=[0;CAchg_SIM];
% Leverage indicator
Lev_SIM=bpSIM./Y_SIM;  
LevSP_SIM=bpSP_SIM./YSP_SIM;
% Real exchange rate
RER_SIM=(omega^(1/(1+ita))+(1-omega)^(1/(1+ita)).*pSIM.^(ita/(1+ita))).^((-1-ita)/ita);
RERSP_SIM=(omega^(1/(1+ita))+(1-omega)^(1/(1+ita)).*pSP_SIM.^(ita/(1+ita))).^((-1-ita)/ita);

%%%%%%% Parameter %%%%%%%%%%%%%%%%%%%%
nbd=3; % periods before and after crisis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;
clear SS
SS=zeros(T,1);      % Indicator for sudden stop
SS_SP=zeros(T,1);   % Indicator for sudden stop under Social Planner's regime

% Crisis is defined as current account goes 2 sd away and collateral
% constraint binds
SS(CA_SIM>(mean(CA_SIM)+2*std(CA_SIM)) & bindSIM>0)=1; 
SS_SP(CASP_SIM>(mean(CA_SIM)+2*std(CA_SIM)) & bindSP_SIM>0)=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Prints results. 
fprintf(' \n ');
fprintf('The long-run probability of crisis in DE is: ');
fprintf('%5.4f \n ',  mean(SS));    
fprintf('The long-run probability of crisis in SP is: ');
fprintf('%5.4f \n ',  mean(SS_SP));

fprintf('The long-run probability of bingding constraint in DE is: ');
fprintf('%5.4f \n ',  mean(bindSIM));
fprintf('The long-run probability of bingding constraint in SP is: ');
fprintf('%5.4f \n ',  mean(bindSP_SIM));

fprintf('std of CA/Y DE is: ');
fprintf('%5.4f \n ',  std(CA_SIM));

fprintf('std of CA/Y SP is: ');
fprintf('%5.4f \n ',  std(CASP_SIM));

fprintf('mean debt/Y level in DE is: ');
fprintf('%5.4f \n ',  mean(Lev_SIM));

fprintf('mean debt/Y level in SP is: ');
fprintf('%5.4f \n ',  mean(LevSP_SIM));

fprintf('welfare gain is: ');
fprintf('%.6f \n ',  mean(WfSIM));
%%%%%%%%%%%%

% Total Consumption, both regimes
CtSIM=(omega.*cSIM.^(-ita)+(1-omega)).^(-1/ita);
CtSPSIM=(omega.*cSP_SIM.^(-ita)+(1-omega)).^(-1/ita);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Normalized deviations: 
    %of total consumption
Cdev=(CtSIM-mean(CtSIM))./mean(CtSIM);
CSPdev=(CtSPSIM-mean(CtSPSIM))./mean(CtSPSIM);
    % of Real Exchange Rate
RERdev=(RER_SIM-mean(RER_SIM))./mean(RER_SIM);
RERSPdev=(RERSP_SIM-mean(RERSP_SIM))./mean(RERSP_SIM);
    % of total output
Ydev=(Y_SIM-mean(Y_SIM))./mean(Y_SIM);  
YSPdev=(YSP_SIM-mean(YSP_SIM))./mean(YSP_SIM);
    % of Tradables output
yTdev=(ySIM-mean(ySIM))./mean(ySIM);

%% 4. Sudden Stop Moments

clear SS_Index SScontrol
% Find Sudden Stop event indices
SS_Index=find(SS==1);               
SSSP_Index=find(SS_SP==1);
%Take out some events that might cause before/after periods go out of bounds
SS_Index=SS_Index(1+nbd:end-nbd); 

temp=SS_Index(2:end)-SS_Index(1:end-1);     % Distance between consecutive SS
temp1=[SS_Index(1);temp];                   % Distance to previous SS
temp=[temp;SS_Index(end)];                  % Distance to next SS
throw=find(temp<=nbd | temp1<=nbd);
% When two SS events are very close (less than nbd), they are the same,
% discard the second one.
throw=find(temp1<=nbd);                     % Identify those close SS                     
SS((SS_Index(throw)))=0;                    % Change value of SS variable
SS_Index(throw)=[];                         % Discard from SS indices vector
SS=min(SS,0);                               % Clear SS vector
SS(SS_Index)=1;                             % New SS vector is 1 only at SS_index

% Track all the exogeneous states that occuered in SSs
% Recall S_index is just exogenous state index  
% Find entries in S_index that show in SS_Index
SScontrol=ismember(S_index,S_index(SS_Index)); 
% Find indices
SScontrol_Index=find(SScontrol==1);

% Financial amplification coefficients 
% Check when SS exogenous states happened but constraint was NOT binding
SSomega=(SScontrol==1 & SS==0);%(SScontrol==1 & bindSIM==0);
SSomega_Index=find(SSomega==1);

% Check when SS exogenous states happened, there was a Curren Account 
% reversal and constraint was binding. These are events where responses are
% amplified by the debt constraint
SSomegaNume=(SScontrol==1 & CA_SIM>(mean(CA_SIM)+2*std(CA_SIM)) & bindSIM>0);
SSomegaNume_Index=find(SSomegaNume==1);

% Deviations to mean contingent on SS
CdevSS=Cdev(SS_Index);
RERdevSS=RERdev(SS_Index);
CSPdevSS=CSPdev(SS_Index);
RERSPdevSS=RERSPdev(SS_Index);


% Print more results
fprintf('C drop in DE is: ');
fprintf('%5.4f \n ',  mean(CdevSS));
fprintf('C drop in SP is: ');
fprintf('%5.4f \n ',  mean(CSPdevSS));
fprintf('RER depreciation in DE is: ');
fprintf('%5.4f \n ',  mean(RERdevSS));
fprintf('RER depreciation in SP is: ');
fprintf('%5.4f \n ',  mean(RERSPdevSS));
fprintf('CA/Y | SS in DE is: ');
fprintf('%5.4f \n ',  mean(CA_SIM(SS_Index)));
fprintf('CA/Y | SS in SP is: ');
fprintf('%5.4f \n ',  mean(CASP_SIM(SS_Index))); 
fprintf('yT drop | SS in DE is: ');
fprintf('%5.4f \n ',  mean(yTdev(SS_Index)));
fprintf('yT drop | SS in SP is: ');
fprintf('%5.4f \n ',  mean(yTdev(SSSP_Index)));

% Financial amplificator coefficient: Average variable drop with binding 
% constraint / avg variable drop with NON-binging constraint
fprintf('Omega C in DE is: ');
fprintf('%5.4f \n ',  mean(Cdev(SSomegaNume_Index))/mean(Cdev(SSomega_Index)));
fprintf('Omega C in SP is: ');
fprintf('%5.4f \n ',  mean(CSPdev(SSomegaNume_Index))/mean(CSPdev(SSomega_Index)));
fprintf('Omega RER in DE is: ');
fprintf('%5.4f \n ',  mean(RERdev(SSomegaNume_Index))/mean(RERdev(SSomega_Index)));
fprintf('Omega RER in SP is: ');
fprintf('%5.4f \n ',  mean(RERSPdev(SSomegaNume_Index))/mean(RERSPdev(SSomega_Index)));
fprintf('Omega CA/Y | SS in DE is: ');
fprintf('%5.4f \n ',  mean(CA_SIM(SSomegaNume_Index))-mean(CA_SIM(SSomega_Index)));
fprintf('Omega CA/Y | SS in SP is: ');
fprintf('%5.4f \n ',  mean(CASP_SIM(SSomegaNume_Index))-mean(CASP_SIM(SSomega_Index))); 

%% 5. Sudden stop event analysis

% Number of events 
nE=length(SS_Index);

% For each event identify variables nbd periods before and after
for i=1:nE
    % States
    State_Event(i,:)=S_index(SS_Index(i)-nbd:SS_Index(i)+nbd);
    % Binding constraint
    bind_Event(i,:)=bindSIM(SS_Index(i)-nbd:SS_Index(i)+nbd);    
    % Interest rate
    R_EventT(i,:)=RSIM(SS_Index(i)-nbd:SS_Index(i)+nbd);
    % Output
    y_EventT(i,:)=ySIM(SS_Index(i)-nbd:SS_Index(i)+nbd);
    % Nontradables Price
    p_EventT(i,:)=(pSIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(pSIM))./mean(pSIM);    
    % Current Acount
    CA_EventT(i,:)=CA_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);    
    % Next period debt
    bp_EventT(i,:)=(bpSIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(bpSIM))./mean(bpSIM); 
    % Consumption
    c_EventT(i,:)=(cSIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(cSIM))./mean(cSIM);
    % Real exchange rate
    RER_EventT(i,:)=RER_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd); 
    % Leverage
    Lev_EventT(i,:)=Lev_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);    
    % Tax
    Tau_EventT(i,:)=Tau_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);   
    % Analogous variables under Social planner's
    bindSP_Event(i,:)=bindSP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);
    pSP_EventT(i,:)=(pSP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(pSP_SIM))./mean(pSP_SIM); 
    CASP_EventT(i,:)=CASP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);
    bpSP_EventT(i,:)=(bpSP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(bpSP_SIM))./mean(bpSP_SIM);
    cSP_EventT(i,:)=(cSP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(cSP_SIM))./mean(cSP_SIM);
    RERSP_EventT(i,:)=RERSP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);
    LevSP_EventT(i,:)=LevSP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);     
end

% Determine which interest rate was prevailing on eahc event
Rh_Event=zeros(size(State_Event));
Rh_Event(State_Event<=NSS)=1;       % For low index states R is high
Rl_Event=1-Rh_Event;                % Al others had low R
Rh_Event=mean(Rh_Event);
Rl_Event=mean(Rl_Event);

%%%%%%%%%%%%%%%%%%%
% Event means
R_Event=mean(R_EventT);
y_Event=mean(y_EventT);
p_Event=mean(p_EventT);
CA_Event=mean(CA_EventT);
bp_Event=mean(bp_EventT);
c_Event=mean(c_EventT);
Tau_Event=mean(Tau_EventT);

Lev_Event=mean(Lev_EventT);

pSP_Event=mean(pSP_EventT);
CASP_Event=mean(CASP_EventT);
bpSP_Event=mean(bpSP_EventT);
cSP_Event=mean(cSP_EventT);
LevSP_Event=mean(LevSP_EventT);
%%%%%%%%%%%%%%%

fprintf('Avg Tax before SS is: ');
fprintf('%5.4f \n ',  Tau_Event(nbd)*100);
fprintf('====================== \n ');
fprintf(' mean tax %5.4f \n ', mean(Tau_SIM)*100 );
fprintf('mean welfare %.6f \n ',  mean(WfSIM)*100 );

% Determinte output and news on each event
yt_Event=zeros(size(State_Event));
ytB=find(yt(1,:)==Z(1));            % Bad output 
ytA=find(yt(1,:)==Z(2));            % Average output
ytG=find(yt(1,:)==Z(3));            % Good output

news_Event=zeros(size(State_Event));
newsB=[1,2,3];                      % Bad news indices
newsB=[newsB,newsB+NSS];
newsA=[4:6];                        % Average news indices
newsA=[newsA,newsA+NSS];
newsG=[7:9];                        % Good news indices
newsG=[newsG,newsG+NSS];

for i=1:length(newsB)
    news_Event(State_Event==newsB(i))=1;
    news_Event(State_Event==newsA(i))=2;
    news_Event(State_Event==newsG(i))=3;
    yt_Event(State_Event==ytB(i))=1;
    yt_Event(State_Event==ytA(i))=2;
    yt_Event(State_Event==ytG(i))=3;
end

% News event averages
news_EventG=sum(news_Event==3)/length(news_Event);
news_EventA=sum(news_Event==2)/length(news_Event);
news_EventB=sum(news_Event==1)/length(news_Event);
% Output event averages
yt_EventG=sum(yt_Event==3)/length(yt_Event);
yt_EventA=sum(yt_Event==2)/length(yt_Event);
yt_EventB=sum(yt_Event==1)/length(yt_Event);

% Event period grid
Event=[-nbd:1:nbd];

%% 6. Plots: just makes figures
margin=0.01;
plotgrid=500;
run plot_cust % Program with stlye parameters for figures

%% Ergodic
jump=[max(bpSIM)-min(bpSIM)+2*margin]/(plotgrid);
pic=figure;
[fk,xik] = ksdensity(bpSIM);
densityk=fk/sum(sum(fk));
H=plot(xik,densityk); 
set(H,'Color',col_de,'Linewidth',LineW_d)
hXlabel=xlabel('Bond Holdings');
hYlabel=ylabel('Frequency');
set(gca,'FontSize',13,'FontName','Times New Roman')

hold on

[fk,xik] = ksdensity(bpSP_SIM);
densityk=fk/sum(sum(fk));
H=plot(xik,densityk,'-.'); 
hLegend=legend('DE','SP','location','northeast');
legend boxoff
xlim([min(bpSIM)-margin,max(bpSIM)]);
set(H,'Color',col_sp,'Linewidth',LineW_d)
set(gca,  'Box'         , 'off'     , 'TickDir'     , 'out'     ,'TickLength'  , [.02 .02] , 'YMinorTick'  , 'off     ' , 'YGrid' , 'off'      , 'XColor'      , [.3 .3 .3],    'YColor'      , [.3 .3 .3],    'LineWidth'   , 1      )  
set(hXlabel,'Fontsize',10,'FontName'   , 'AvantGarde')    
set(hYlabel,'Fontsize',15,'FontName'   , 'AvantGarde')   
set(hLegend,'Fontsize',20,'FontName'   , 'AvantGarde') 
filename=sprintf('Ergodic_bond_info_%1.2f.eps',info);

%%Event plots
pic=figure;
subplot(3,1,1)

H=bar(Event, [news_EventG' news_EventA' news_EventB'] , 0.2, 'stack');
hTitle=title('Exogeneous States around Crisis');
hYlabel1=ylabel('Frequency');
hLegend1=legend('Good News', 'Avg News', 'Bad News');
hXlabel1=xlabel('T');

subplot(3,1,2)

H=bar(Event, [Rh_Event' Rl_Event'] , 0.2, 'stack');
hYlabel2=ylabel('Frequency');
hLegend2=legend('High R', 'LOW R');
hXlabel2=xlabel('T');

subplot(3,1,3)
H=bar(Event, [yt_EventG' yt_EventA' yt_EventB'] , 0.2, 'stack');
hYlabel3=ylabel('Frequency');
hLegend3=legend('Good yT', 'Avg yT', 'Bad yT');
hXlabel3=xlabel('T');

set(hTitle,'Fontsize',15,'FontName'   , 'AvantGarde')
set([hYlabel1 hYlabel2 hYlabel3],'Fontsize',15,'FontName'   , 'AvantGarde')   
set([hXlabel1 hXlabel2 hXlabel3],'Fontsize',15,'FontName'   , 'AvantGarde')   
set([hLegend1 hLegend2 hLegend3],'Fontsize',5,'FontName'   , 'AvantGarde') 

pic=figure;
subplot(2,2,1)

H1=plot(Event,p_Event,Event,pSP_Event,'-.'); 
hTitle1=title('Pn');
hLegend1=legend('DE', 'SP');
hXlabel1=xlabel('T');
hYlabel1=ylabel('Deviation from mean');

subplot(2,2,2)
H2=plot(Event,bp_Event,Event,bpSP_Event,'-.');
hLegend2=legend('DE', 'SP');
hXlabel2=xlabel('T');
hTitle2=title('B`');
hYlabel2=ylabel('Deviation from mean');

subplot(2,2,3)
H3=plot(Event,c_Event,Event,cSP_Event,'-.');
hLegend3=legend('DE', 'SP');
hXlabel3=xlabel('T');
hTitle3=title('cT');
hYlabel3=ylabel('Deviation from mean');

subplot(2,2,4)
H4=plot(Event,CA_Event,Event,CASP_Event,'-.');
hLegend4=legend('DE', 'SP');
hXlabel4=xlabel('T');
hTitle4=title('CA/Y');
hYlabel4=ylabel('Deviation from mean');
set([H1(1) H2(1) H3(1) H4(1)],'Color',col_de,'Linewidth',LineW_d)
set([H1(2) H2(2) H3(2) H4(2)],'Color',col_sp,'Linewidth',LineW_d)
set(gca,  'Box'         , 'off'     , 'TickDir'     , 'out'     ,'TickLength'  , [.02 .02] , 'YMinorTick'  , 'off     ' , 'YGrid' , 'off'      , 'XColor'      , [.3 .3 .3],    'YColor'      , [.3 .3 .3],    'LineWidth'   , 1      )  
set([hTitle1 hTitle2 hTitle3 hTitle4],'Fontsize',15,'FontName'   , 'AvantGarde')
set([hXlabel1 hXlabel2 hXlabel3 hXlabel4] ,'Fontsize',10,'FontName'   , 'AvantGarde')   
set([hYlabel1 hYlabel2 hYlabel3 hYlabel4] ,'Fontsize',10,'FontName'   , 'AvantGarde') 
set([hLegend1 hLegend2 hLegend3 hLegend4],'Fontsize',10,'FontName'   , 'AvantGarde') 
